package nocturne.jfly;


import java.util.Random;

/**
 *   
 */
public class Fly implements IFly
{
    public static int SPEED = 2;
    public static int SCALE = 3;

    //  
    private int _x = 0;
    private int _y = 0;

    //    
    private float _dx = 0;
    private float _dy = 0;

    //   
    private IEnvironment _environment;

    //   
    private Random _random = new Random();

    //  
    private PatternManager _patternManager = new PatternManager();

    public Fly(IEnvironment _environment)
    {
        this._environment = _environment;

        _x = _environment.getFieldWidth() / 2;
        _y = _environment.getFieldHeight() / 2;

        _patternManager.get_patterns().add(new VectorPattern(new int[] {1, 1, 1, 1, 1, 2, 2, -1, -2, -3, -1, 0, 0, 1, 2, 2, -2, -2, -1, 0},
                                                             new int[] {0, 0, 0, 0, -1, -1, -1, -1, -1, 0, 0, 0, 0, 0, 2, 2, 2, 2, 2, 2}));
        _patternManager.get_patterns().add(new VectorPattern(new int[] {0, 0, 1, 2, 3, 4, 5, 4, 3, 2, 1, 3, 3, 3, 3, 2, 1, -2, -2, -1},
                                                             new int[] {1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 0, 0, 0, 0}));
        _patternManager.get_patterns().add(new VectorPattern(new int[] {0, -1, -2, -3, -3, -2, -2, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1},
                                                             new int[] {1, 1, 1, 2, 2, -1, -1, -1, -2, -2, -1, -1, 0, 0, 0, 1, 1, 1, 1, 1}));
    }

    //      
    public int getDistance()
    {
        return (int)(Math.sqrt((_x - _environment.getPlayerX()) * (_x - _environment.getPlayerX()) +
                              (_y - _environment.getPlayerY()) * (_y - _environment.getPlayerY())) / SCALE);
    }

    /**
     *   
     */
    public boolean isCurrentPatternFinished()
    {
        return _patternManager.get_currentPattern().isFinished();
    }

    /**
     *   
     */
    public void moveFromPlayer()
    {
        LoggingEnvironment.getInstance().get_log().log("Fly", "moveFromPlayer:   ", IFlyLog.NOTIFICATION);


        if (_x < _environment.getPlayerX())
            _x -= SPEED;
        else if (_x > _environment.getPlayerX())
            _x += SPEED;

        if (_y < _environment.getPlayerY())
            _y -= SPEED;
        else if (_y > _environment.getPlayerY())
            _y += SPEED;

        checkPosition();
    }

    /**
     *   .
     */
    public void moveToPlayer()
    {
        LoggingEnvironment.getInstance().get_log().log("Fly", "moveToPlayer:   ", IFlyLog.NOTIFICATION);

        if (_x < _environment.getPlayerX())
            _x += SPEED;
        else if (_x > _environment.getPlayerX())
            _x -= SPEED;

        if (_y < _environment.getPlayerY())
            _y += SPEED;
        else if (_y > _environment.getPlayerY())
            _y -= SPEED;

        checkPosition();
    }

    /**
     *   ,   
     */
    public void moveByDirectionVector()
    {
        LoggingEnvironment.getInstance().get_log().log("Fly", "moveByDirectionVector:    ", IFlyLog.NOTIFICATION);

        _x += _dx;
        _y += _dy;

        checkPosition();
    }

    /**
     *   .
     */
    public void setRandomDirection()
    {
        LoggingEnvironment.getInstance().get_log().log("Fly", "setRandomDirection:   ", IFlyLog.NOTIFICATION);

        _dx = _random.nextFloat() * SPEED;
        _dy = _random.nextFloat() * SPEED;
    }

    /**
     *   
     */
    public void setRandomPattern()
    {
        _patternManager.selectRandom();
    }

    /**
     *     
     */
    public void goToNextPatternPosition()
    {
        _x += _patternManager.get_currentPattern().get_x() * SPEED;
        _y += _patternManager.get_currentPattern().get_y() * SPEED;

        _patternManager.get_currentPattern().next();
    }

    public int get_x()
    {
        return _x;
    }

    public int get_y()
    {
        return _y;
    }

    // ,     
    private void checkPosition()
    {
        if (_x < 0)
            _x = 0;
        if (_y < 0)
            _y = 0;

        if (_x >= _environment.getFieldWidth())
            _x = _environment.getFieldWidth() - 1;

        if (_y >= _environment.getFieldHeight())
            _y = _environment.getFieldHeight() - 1;
    }
}
