package nocturne.jfly;

import java.applet.Applet;
import java.awt.*;
import java.awt.image.ImageFilter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Timer;
import java.util.TimerTask;

/**
 *  
 */
public class FlyApplet extends Applet implements MouseMotionListener
{
    private Environment _environment;
    private Fly _fly;
    private Automata _automata;
    private Image _canvas;
    private Timer _timer = new Timer();
    private MediaTracker _tracker;
    private Font _normalFont = new Font("Arial", Font.PLAIN, 11);
    private Font _boldFont = new Font("Arial", Font.BOLD, 11);

    /*
     * Images for drawing fly
     */
    private Image _redFly, _greenFly, _blueFly, _yellowFly, _greyFly;

    public void init()
    {
        super.init();

        _environment = new Environment(getWidth(), getHeight(), 0, 0);
        _fly = new Fly(_environment);
        _automata = new Automata(_fly, new RandomGenerator(), new TickCounter());

        _canvas = this.createImage(getWidth(), getHeight());

        _tracker = new MediaTracker(this);
        _redFly = getImage(getDocumentBase(), "img/fly_red.gif");
        _blueFly = getImage(getDocumentBase(), "img/fly_blue.gif");
        _greenFly = getImage(getDocumentBase(), "img/fly_green.gif");
        _yellowFly = getImage(getDocumentBase(), "img/fly_yellow.gif");
        _greyFly = getImage(getDocumentBase(), "img/fly.gif");

        _tracker.addImage(_redFly, 0);
        _tracker.addImage(_blueFly, 1);
        _tracker.addImage(_greenFly, 2);
        _tracker.addImage(_yellowFly, 3);
        _tracker.addImage(_greyFly, 4);

        this.addMouseMotionListener(this);
    }

    public void start()
    {
        super.start();

        try
        {
            _tracker.waitForAll();
        }
        catch (InterruptedException e)
        {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }

        _timer.scheduleAtFixedRate(new ActionTimerTask(), 100, 50);
    }

    public void stop()
    {
        _timer.cancel();

        super.stop();
    }

    public void resize(Dimension d)
    {
        super.resize(d);
    }


    public void mouseDragged(MouseEvent e)
    {
    }

    public void mouseMoved(MouseEvent e)
    {
        _environment.setPlayerX((int)e.getPoint().getX());
        _environment.setPlayerY((int)e.getPoint().getY());
    }

    public void update(Graphics g)
    {
        g.drawImage(_canvas, 0, 0, this);
    }

    public void paint(Graphics g)
    {
        update(g);
    }

    private void paintFly()
    {
        Graphics g = _canvas.getGraphics();

        g.setColor(Color.WHITE);
        g.fillRect(0, 0, _canvas.getWidth(this), _canvas.getHeight(this));
        g.setColor(Color.BLACK);
        g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);

        //  
        Image flyImage = null;

        switch (_automata.get_y0())
        {
        case 0:
            flyImage = _greyFly;
            break;
        case 1:
            flyImage = _blueFly;
            break;
        case 2:
            flyImage = _redFly;
            break;
        case 3:
            flyImage = _greenFly;
            break;
        case 4:
            flyImage = _yellowFly;
            break;
        }

        int x = _fly.get_x() - flyImage.getWidth(this) / 2;
        int y = _fly.get_y() - flyImage.getHeight(this) / 2;

        g.drawImage(flyImage, x, y, this);

        //  
        int imagey = 10;


        imagey = drawFly(g, flyImage, _redFly, " \"\"", imagey);
        imagey = drawFly(g, flyImage, _blueFly, " \"\"", imagey);
        imagey = drawFly(g, flyImage, _greenFly, " \" \"", imagey);
        imagey = drawFly(g, flyImage, _yellowFly, " \"  \"", imagey);
        imagey = drawFly(g, flyImage, _greyFly, " \" \"", imagey);
    }

    private int drawFly( Graphics g, Image flyImage, Image sampleImage, String text, int y )
    {
        g.drawImage(sampleImage, 10, y, this);
        y += sampleImage.getHeight(this) + 10;

        if (flyImage.equals(sampleImage))
            g.setFont(_boldFont);
        else
            g.setFont(_normalFont);

        g.drawString(text, 10 + sampleImage.getWidth(this) + 5, y - 20);

        return y;
    }

    /**
     * ,  
     */
    public class ActionTimerTask extends TimerTask
    {
        /**
         *     .
         */
        public void run()
        {
            _automata.A0(1);

            paintFly();
            update(getGraphics());
        }
    }
}
